let string_of_category = function
    `godi -> "godi"
  | `conf -> "conf"
  | `apps -> "apps"
  | `base -> "base"

let string_of_relop = function
  | `lt -> "<"
  | `gt -> ">"
  | `le -> "<="
  | `ge -> ">="
  | `eq -> "=="

let string_of_dep = function
    | (cat, name, None) ->
        (string_of_category cat) ^ "-" ^ name ^ "-[0-9]*:../../"
        ^ (string_of_category cat) ^ "/" ^ (string_of_category cat) ^ "-" ^ name
    | (cat, name, Some (relop, version)) ->
        (string_of_category cat) ^ "-" ^ name ^ (string_of_relop relop)
             ^ version ^ ":../../"
             ^ (string_of_category cat) ^ "/"
             ^ (string_of_category cat) ^ "-" ^ name

let f il =
    let catstring = string_of_category il.IL.category in
    let catname = catstring ^ "-" ^ il.IL.name in
    {
      Godi_package.specfile = il.IL.specfile;
      Godi_package.makefile = {
        Godi_package.version = il.IL.version;
        Godi_package.pkgname = catname ^ "-${VERSION}";
        Godi_package.revision = string_of_int il.IL.revision;
        Godi_package.distname = il.IL.sources_unpacksto;
        Godi_package.distfiles =
            il.IL.sources_basename ^ il.IL.sources_extension;
        Godi_package.categories = catstring;
        Godi_package.master_sites = il.IL.sources_site ^ "/";
        Godi_package.maintainer = il.IL.maintainer;
        Godi_package.homepage = il.IL.homepage;
        Godi_package.comment = il.IL.short_desc;
        Godi_package.confopt_defaults =
            List.fold_left (fun acc confopt ->
                ( acc ^ confopt.Common.name
                ^ "?=" ^ confopt.Common.default
                ^ "\n"))
                ""
                il.IL.confopts;
        Godi_package.pkg = catname;
        Godi_package.addl_make_flags =
            List.fold_left (fun acc confopt ->
                match confopt.Common.implementation with
                  `makevar s ->
                    acc ^ " " ^ s ^ "=\"$(" ^ confopt.Common.name ^ ")\""
                 | _ -> acc)
               ""
               il.IL.confopts;
        Godi_package.depends =
            String.concat "\n"
                          (List.map (fun d -> "DEPENDS+= "
                                            ^ (string_of_dep d))
                                    il.IL.depends);
        Godi_package.build_depends =
            String.concat "\n"
                          (List.map (fun d -> "BUILD_DEPENDS+= "
                                            ^ (string_of_dep d))
                                    il.IL.build_depends);
        Godi_package.has_configure =
            if List.mem `configure il.IL.options then
               ( "HAS_CONFIGURE = yes\n"
               ^ "CONFIGURE_ARGS+= --prefix ${PREFIX}\n"
               ^ "CONFIGURE_ENV+= ${BUILD_OCAMLFIND_ENV}")
            else
                "# HAS_CONFIGURE = yes";
        Godi_package.addl_configure_args =
            List.fold_left (fun acc confopt ->
                match confopt.Common.implementation with
                  `configarg s -> acc ^ "CONFIGURE_ARGS+= " ^ s ^ "\n"
                 | _ -> acc)
               ""
               il.IL.confopts;
        Godi_package.use_gmake =
            if List.mem `bmake il.IL.options then
                "# USE_GMAKE = yes"
            else
                "USE_GMAKE = yes";
        Godi_package.all_target = il.IL.all_target;
        Godi_package.all_target_opt =
            if List.mem `opt il.IL.options then
                "ALL_TARGET+= " ^ il.IL.opt_target
            else
                "# ALL_TARGET+= " ^ il.IL.opt_target;
        Godi_package.docfiles = String.concat " " il.IL.docfiles;
      };
      Godi_package.descr = Util.textwrap il.IL.long_desc Config.linewrap;
      Godi_package.buildmsg =
        (match il.IL.confopts with
            [] -> None
          | opts ->
              let desc = List.fold_left
                (fun acc co ->
                   acc ^ co.Common.name ^ " - " ^ co.Common.description ^ "\n")
                ""
                opts
              in
              let desc = Util.textwrap desc Config.linewrap in
              Some desc);
      Godi_package.confopts =
        (match il.IL.confopts with
            [] -> None
          | opts -> Some (List.fold_left
                            (fun acc co -> acc ^ co.Common.name ^ "\n")
                            ""
                            opts));
      Godi_package.patches = il.IL.patches;
      Godi_package.filesdir = il.IL.filesdir;
    }
